import torch

from allennlp.common.testing import AllenNlpTestCase, multi_device

actual_devices = set()


class TestTesting(AllenNlpTestCase):
    @multi_device
    def test_multi_device(self, device: str):
        actual_devices.add(device)

    def test_devices_accounted_for(self):
        expected_devices = {"cpu", "cuda"} if torch.cuda.is_available() else {"cpu"}
        assert expected_devices == actual_devices



import numpy as np
import pytest
import torch
from numpy.testing import assert_almost_equal

from allennlp.common.checks import ConfigurationError
from allennlp.common.testing import AllenNlpTestCase
from allennlp.data import Vocabulary
from allennlp.modules.token_embedders import BagOfWordCountsTokenEmbedder


class TestBagOfWordCountsTokenEmbedder(AllenNlpTestCase):
    def setup_method(self):
        super().setup_method()
        self.vocab = Vocabulary()
        self.vocab.add_token_to_namespace("1")
        self.vocab.add_token_to_namespace("2")
        self.vocab.add_token_to_namespace("3")
        self.vocab.add_token_to_namespace("4")
        self.non_padded_vocab = Vocabulary(non_padded_namespaces=["tokens"])

    def test_forward_calculates_bow_properly(self):
        embedder = BagOfWordCountsTokenEmbedder(self.vocab)
        numpy_tensor = np.array([[2, 0], [3, 0], [4, 4]])
        inputs = torch.from_numpy(numpy_tensor)
        embedder_output = embedder(inputs)
        numpy_tensor = np.array([[0, 0, 1, 0, 0, 0], [0, 0, 0, 1, 0, 0], [0, 0, 0, 0, 2, 0]])
        manual_output = torch.from_numpy(numpy_tensor).float()
        assert_almost_equal(embedder_output.data.numpy(), manual_output.data.numpy())

    def test_zeros_out_unknown_tokens(self):
        embedder = BagOfWordCountsTokenEmbedder(self.vocab, ignore_oov=True)
        numpy_tensor = np.array([[1, 5], [2, 0], [4, 4]])
        inputs = torch.from_numpy(numpy_tensor)
        embedder_output = embedder(inputs)
        numpy_tensor = np.array([[0, 0, 0, 0, 0, 1], [0, 0, 1, 0, 0, 0], [0, 0, 0, 0, 2, 0]])
        manual_output = torch.from_numpy(numpy_tensor).float()
        assert_almost_equal(embedder_output.data.numpy(), manual_output.data.numpy())

    def test_ignore_oov_should_fail_on_non_padded_vocab(self):
        with pytest.raises(ConfigurationError):
            BagOfWordCountsTokenEmbedder(self.non_padded_vocab, ignore_oov=True)

    def test_projects_properly(self):
        embedder = BagOfWordCountsTokenEmbedder(vocab=self.vocab, projection_dim=50)
        numpy_tensor = np.array([[1, 0], [1, 0], [4, 4]])
        inputs = torch.from_numpy(numpy_tensor)
        embedder_output = embedder(inputs)
        assert embedder_output.shape[1] == 50



import torch

from allennlp.modules.token_embedders.embedding import Embedding
from allennlp.modules.seq2vec_encoders.seq2vec_encoder import Seq2VecEncoder
from allennlp.modules.time_distributed import TimeDistributed
from allennlp.modules.token_embedders.token_embedder import TokenEmbedder


@TokenEmbedder.register("character_encoding")
class TokenCharactersEncoder(TokenEmbedder):
    """
    A `TokenCharactersEncoder` takes the output of a
    [`TokenCharactersIndexer`](../../data/token_indexers/token_characters_indexer.md), which is a tensor of shape
    (batch_size, num_tokens, num_characters), embeds the characters, runs a token-level encoder, and
    returns the result, which is a tensor of shape (batch_size, num_tokens, encoding_dim).  We also
    optionally apply dropout after the token-level encoder.

    We take the embedding and encoding modules as input, so this class is itself quite simple.

    Registered as a `TokenEmbedder` with name "character_encoding".
    """

    def __init__(self, embedding: Embedding, encoder: Seq2VecEncoder, dropout: float = 0.0) -> None:
        super().__init__()
        self._embedding = TimeDistributed(embedding)
        self._encoder = TimeDistributed(encoder)
        if dropout > 0:
            self._dropout = torch.nn.Dropout(p=dropout)
        else:
            self._dropout = lambda x: x

    def get_output_dim(self) -> int:
        return self._encoder._module.get_output_dim()

    def forward(self, token_characters: torch.Tensor) -> torch.Tensor:
        mask = (token_characters != 0).long()
        return self._dropout(self._encoder(self._embedding(token_characters), mask))
